/* copyright (C) 2016 Sohta. */

#define APP_VERSION		0x0914
#define APP_VERSION_T	"0914"

#include <stdio.h>
#include <stdlib.h>
#include <system.h>
#include <system_config.h>

#include <usb.h>
#include <usb_device.h>
#include <usb_device_cdc.h>
#include <BootPIC18NonJ.h>

const uint16_t __at(APP_VERSION_ADDRESS) FlashVersion = APP_VERSION;

static uint16_t sntpYear;
static uint8_t sntpMonth;
static uint8_t sntpDay;
static uint8_t sntpWeek;
static uint16_t sntpDays;
static uint32_t sntpSec;

enum {
	SNTP_INVALID	= 0,
	SNTP_UPDATED,
	SNTP_NOTUPDATED,
};
static uint8_t sntpStatus;

enum {
	DISP_HourMin = 0,
	DISP_MonthDay,
	DISP_Toggle,
	DISP_Year,
	DISP_Week,
	DISP_Sec,

	DISP_EORZEA_HourMin = 10,
	DISP_EORZEA_MonthDay,
	DISP_EORZEA_Toggle,
	DISP_EORZEA_Year,
	DISP_EORZEA_Week,

	DISP_BAT = 0x10,
	DISP_AD,
	DISP_GPIO,

	DISP_HTTP = 0x20,
};
static uint8_t dispMode;

enum {
	POST_OFF = 0,
	POST_BAT,
	POST_AD,
	POST_GPIO,
};
static uint8_t postMode;

static uint8_t HttpMes[8];
static uint8_t HttpMesBlink;
static uint16_t HttpMesNext;

       uint8_t timeoutCnt;
static uint8_t timeoutWlan;
static uint8_t reqLowbat;

static uint8_t gpio_last;
static uint8_t gpio_cur;

#define TIMEOUT_WLAN  180

enum {
	STATUS_0_BOOTING	= 0,
	STATUS_1_WPSSTART	= 1,	//    wps started
	STATUS_2_CONNECTING	= 2,	// S2:wps success,connecting ap ...
	STATUS_3_CONNECTED	= 3,	// S3:WIFI CONNECTED
	STATUS_4_GOTIP		= 4,	// S4:WIFI GOT IP
	STATUS_5_SNTP		= 5,	// S5:2016/09/04 1 11:12:28 17048 40348
	STATUS_6_ERROR		= 6,	// ERROR, wps fail
	STATUS_7_HTTP		= 7,	// S7:
	STATUS_8_MODE		= 8,	// S8:0 0

	STATUS_IDLE,				// 
	STATUS_WLANBOOT,			// WLAN boot
	STATUS_WLANOFF,
};
static uint8_t wifiStatus;

static uint8_t lastLineIndex = 0;

enum {
	ERROR_NONE			= 0,
	ERROR_WPS			= 1,
	ERROR_CONNECTION	= 2,
	ERROR_GOTIP			= 3,
	ERROR_SNTP			= 4,

	ERROR_WLANFW		= 5,

	ERROR_7_HTTP		= 7,
};
static uint8_t errorCode = 0;


//------------------------------------------------------------
// CONFIG1L
#pragma config CPUDIV	= NOCLKDIV	// CPU system clock selection
									//		00 = 1/1
									//		01 = 1/2
									//		10 = 1/3
									//		11 = 1/4
#pragma config USBDIV	= OFF		// USB clock selection
									//		0 = USB clock = OSC
									//		1 = USB clock = OSC/2

// CONFIG1H
#pragma config FOSC		= HS		// oscillator selection
									//		0000 = LP oscillator
									//		0001 = XT oscillator
									//		0010 = HS oscillator
									//		0011 = external RC oscillator, CLKOUT function on OSC2
									//		0100 = EC, CLKOUT function on OSC2(high)
									//		0101 = EC(high)
									//		0111 = external RC oscillator
									//		1000 = internal RC oscillator
									//		1001 = internal RC oscillator, CLKOUT function on OSC2
									//		1010 = EC, CLKOUT function on OSC2(medium)
									//		1011 = EC(medium)
									//		1100 = EC, CLKOUT function on OSC2(low)
									//		1101 = EC(low)
#pragma config PLLEN	= OFF		// 4 x pll enable
#pragma config FCMEN	= OFF		// fall-safe clock monitor enable
#pragma config IESO		= OFF		// internal/external oscillator switchover

// CONFIG2L
#pragma config PWRTEN	= OFF		// power-up timer enable,  OFF=PWRT enabled
#pragma config BOREN	= OFF		// brown-out reset enable
#pragma config BORV		= 30		// brown-out reset voltage

// CONFIG2H
#pragma config WDTEN	= OFF		// watchdog timer enable
#pragma config WDTPS	= 32768		// watchdog timer postscale select

// CONFIG3H
#pragma config HFOFST	= OFF		// HFINTOSC fast start-up bit
#pragma config MCLRE	= OFF		// MCLR pin enable

// CONFIG4L
#pragma config STVREN	= ON		// stack full/underflow reset enable
#pragma config LVP		= OFF
#pragma config XINST	= OFF
#pragma config BBSIZ	= OFF

// CONFIG5L
#pragma config CP0		= OFF
#pragma config CP1		= OFF

// CONFIG5H
#pragma config CPB		= OFF

// CONFIG6L
#pragma config WRT0		= OFF
#pragma config WRT1		= OFF

// CONFIG6H
#pragma config WRTB		= OFF
#pragma config WRTC		= OFF

// CONFIG7L
#pragma config EBTR0	= OFF
#pragma config EBTR1	= OFF

// CONFIG7H
#pragma config EBTRB	= OFF

//------------------------------------------------------------
static void batteryInit(void)
{
	ADCON0	= (15<<2)				// CHS: analog channel select bits
									//		3~11 =  AN3~AN11
									//		14 = DAC
									//		15 = FVR
			| (0<<1)				// GO_xDONE: AD conversion status bit
									//		1 = AD conversion cycle in progress. setting this bit sstarts an AD conversion cycle.
									//			this bit is automatically cleared by hardware when the AD conversion has completed.
			| (1<<0);				// ADON:ADC Enable bit

	ADCON1	= (0b00<<2)				// PVCFG: positive voltade reference select bit
									//		00 = internally by VDD
									//		01 = externally through VREF+
									//		10 = internally through FVR
			| (0b00<<0);			// NVCFG: negative voltade reference select bit
									//		00 = internally by VSS
									//		01 = externally through VREF-

	ADCON2	= (1<<7)				// ADFM: AD conversion result format select bit
									//		1 = right justified
									//		0 = left justified
			| (0b111<3)				// ACQT: AD acquisition time select bits. acquisition time is the duration that the AD charge
									//		holding capacitor remains connected to AD channel from the instant the GO_xDONE bit is set untile
									//		conversions begins.
									//		000 = 0
									//		001 = 2 TAD
									//		010 = 4 TAD
									//		011 = 6 TAD
									//		100 = 8 TAD
									//		101 = 12 TAD
									//		110 = 16 TAD
									//		111 = 20 TAD
			| (0b111<<0);			// ADCS: AD conversion clock select bits.
									//		000 = FOSC/2
									//		001 = FOSC/8
									//		010 = FOSC/32
									//		011 = FRC (clock derived from a dedicated internal oscillator = 600khz nominal)
									//		100 = FOSC/4
									//		101 = FOSC/16
									//		110 = FOSC/64
									//		111 = FRC (clock derived from a dedicated internal oscillator = 600khz nominal)

	REFCON0	= (1<<7)				// FVR1EN: fixed voltage reference 1 enable
			| (0<<6)				// FVR1ST: fixed voltage reference 1 stable
			| (0b10<<4);			// FVR1S: fixed voltage reference 1 voltage select bits
									//		01 = 1.024V
									//		10 = 2.048V
									//		11 = 4.096V
	while(!REFCON0bits.FVR1ST);
	return;
}

static uint16_t batteryGet(void)	// 0~4092
{
	uint16_t ad;
	ADCON0	= (15<<2)				// CHS: analog channel select bits
									//		15 = FVR
			| (1<<1)				// GO_xDONE: AD conversion status bit
									//		1 = AD conversion cycle in progress. setting this bit sstarts an AD conversion cycle.
									//			this bit is automatically cleared by hardware when the AD conversion has completed.
			| (1<<0);				// ADON:ADC Enable bit

	while(ADCON0bits.GO);
	ad = ADRES;
	return ad ? (2048UL*1048UL)/ad : 0;
}

static void batteryClose(void)
{
	ADCON0 = 0;
	REFCON0bits.FVR1EN = 0;
	return;
}

static uint16_t ad10Get()
{
	uint16_t ad;
	ANSELHbits.ANS10 = 1;			// AN10

	ADCON0	= (10<<2)				// CHS: analog channel select bits
									//		3~11 =  AN3~AN11
									//		14 = DAC
									//		15 = FVR
			| (0<<1)				// GO_xDONE: AD conversion status bit
									//		1 = AD conversion cycle in progress. setting this bit sstarts an AD conversion cycle.
									//			this bit is automatically cleared by hardware when the AD conversion has completed.
			| (1<<0);				// ADON:ADC Enable bit

	ADCON1	= (0b10<<2)				// PVCFG: positive voltade reference select bit
									//		00 = internally by VDD
									//		01 = externally through VREF+
									//		10 = internally through FVR
			| (0b00<<0);			// NVCFG: negative voltade reference select bit
									//		00 = internally by VSS
									//		01 = externally through VREF-

	ADCON2	= (1<<7)				// ADFM: AD conversion result format select bit
									//		1 = right justified
									//		0 = left justified
			| (0b111<3)				// ACQT: AD acquisition time select bits. acquisition time is the duration that the AD charge
									//		holding capacitor remains connected to AD channel from the instant the GO_xDONE bit is set untile
									//		conversions begins.
									//		000 = 0
									//		001 = 2 TAD
									//		010 = 4 TAD
									//		011 = 6 TAD
									//		100 = 8 TAD
									//		101 = 12 TAD
									//		110 = 16 TAD
									//		111 = 20 TAD
			| (0b111<<0);			// ADCS: AD conversion clock select bits.
									//		000 = FOSC/2
									//		001 = FOSC/8
									//		010 = FOSC/32
									//		011 = FRC (clock derived from a dedicated internal oscillator = 600khz nominal)
									//		100 = FOSC/4
									//		101 = FOSC/16
									//		110 = FOSC/64
									//		111 = FRC (clock derived from a dedicated internal oscillator = 600khz nominal)

	REFCON0	= (1<<7)				// FVR1EN: fixed voltage reference 1 enable
			| (0<<6)				// FVR1ST: fixed voltage reference 1 stable
			| (0b10<<4);			// FVR1S: fixed voltage reference 1 voltage select bits
									//		01 = 1.024V
									//		10 = 2.048V
									//		11 = 4.096V
	while(!REFCON0bits.FVR1ST);

	DelayRoutine(10);

	ADCON0	= (10<<2)				// CHS: analog channel select bits
									//		10 = AN10
			| (1<<1)				// GO_xDONE: AD conversion status bit
									//		1 = AD conversion cycle in progress. setting this bit sstarts an AD conversion cycle.
									//			this bit is automatically cleared by hardware when the AD conversion has completed.
			| (1<<0);				// ADON:ADC Enable bit
	while(ADCON0bits.GO);
	ad = ADRES * 2;

	ADCON0 = 0;
	REFCON0bits.FVR1EN = 0;
	ANSELHbits.ANS10 = 0;			// AN10
	return ad;

}
//------------------------------------------------------------

#define SPBRG_48M (48000000/4/115200 - 1)
#define SPBRG_12M (12000000/4/115200 - 1)
//#define SPBRG_48M (48000000/4/ 74850 - 1)
//#define SPBRG_12M (12000000/4/ 74850 - 1)

void USART_Initialize()
{
	volatile uint8_t c;

	TXSTA	= (0<<0)		// TX9D: ninth bit of transmit data
			| (0<<1)		// TRMT: transmit shift register status
			| (1<<2)		// BRGH: high baud rate select
			| (0<<3)		// SENDB: send break character
			| (0<<4)		// SYNC: EUSART mode select
							//		1 = synchronous mode
							//		0 = Asynchronous mode
			| (1<<5)		// TXEN: transmit enable
			| (0<<6)		// TX9: 9-bit transmit enable
			| (0<<7);		// CSRC: clock source select(synchronous mode)

	RCSTA	= (0<<0)		// RX9D: ninth bit of received data
			| (0<<1)		// OERR: overrun error
			| (0<<2)		// FERR: framing error
			| (0<<3)		// ADDEN: address detec enable(RX9=1)
			| (1<<4)		// CREN: continuous receive enable
			| (0<<5)		// SREN: single receive enable(synchronous mode)
			| (0<<6)		// RX9: 9-bit receive enable
			| (1<<7);		// SPEN: serial port enable

	SPBRG  =  SPBRG_12M;
	SPBRGH = (SPBRG_12M>>8);

	BAUDCON = 0x08;			// BRG16 = 1
	c = RCREG;				// read
}

uint8_t USART_getcUSART()
{
	char  c;
	// in case of overrun error we should never see an overrun error, but if we do, reset the port and keep going.
	if (RCSTAbits.OERR) {
		RCSTAbits.CREN = 0;
		c = RCREG;
		RCSTAbits.CREN = 1;
	} else {
		c = RCREG;
	}
	return c;
}

void USART_putcUSART(char c)
{
	TXREG = c;
}

//------------------------------------------------------------
enum {
	MES_CON	= 0,
	MES_ERR	= 1,
	MES_UP	= 2,
};

static void lcdUpdateDate(void)
{
}

static const uint8_t seg7dec[10] = {
	0b11101110,	//0
	0b10000010,	//1
	0b11011100,	//2
	0b11010110,	//3
	0b10110010,	//4
	0b01110110,	//5
	0b01111110,	//6
	0b11100010,	//7
	0b11111110,	//8
	0b11110110,	//9
};

static const uint8_t seg7dec2[10] = {
	0b00000000,	//0
	0b10000010,	//1
	0b11011100,	//2
	0b11010110,	//3
	0b10110010,	//4
	0b01110110,	//5
	0b01111110,	//6
	0b11100010,	//7
	0b11111110,	//8
	0b11110110,	//9
};

static const uint8_t seg7alpha[] = {
//	0b00010000,	//-

	0b11111010,	//A
	0b00111110,	//B
	0b01101100,	//C
	0b10011110,	//D
	0b01111100,	//E
	0b01111000,	//F
	0b01101110,	//G
	0b00111010,	//H
	0b00000010,	//I
	0b10001110,	//J
	0b01111010,	//K
	0b00101100,	//L
	0b01011010,	//M
	0b00011010,	//N
	0b00011110,	//O
	0b11111000,	//P
	0b11110010,	//Q
	0b00011000,	//R
	0b00110110,	//S
	0b00111100,	//T
	0b00001110,	//U
	0b10101110,	//V
	0b01001110,	//W
	0b10101010,	//X
	0b10110110,	//Y
	0b11001100,	//Z
};

static uint8_t toggle;
void spi_data8_2(uint8_t adrs, uint8_t data)
{
	switch(adrs) {
	case 0:
		data |= toggle & reqLowbat;
		break;
	case 2:
		data ^= toggle & ((sntpStatus == SNTP_NOTUPDATED) ? 1 : 0);
		break;
	}
	spi_data8(adrs, data);
}

static void lcdUpdateTime(void)
{
	toggle = (sntpStatus ? sntpSec : timeoutCnt) & 1;
	uint16_t d;

	int mode = dispMode;
	switch(mode) {
	case DISP_Toggle:
		mode = (sntpSec & 4) ? DISP_MonthDay : DISP_HourMin;
		break;
	case DISP_EORZEA_Toggle:
		mode = (sntpSec & 4) ? DISP_EORZEA_MonthDay : DISP_EORZEA_HourMin;
		break;
	}

	if(postMode == POST_GPIO || mode == DISP_GPIO) {
		WPUBbits.WPUB4 = 1;
		DelayRoutine(10);
		gpio_cur = PORTBbits.RB4 ^ 1;
		WPUBbits.WPUB4 = 0;
	}

	if(sntpStatus == SNTP_INVALID) {
		spi_data8_2(0, 0b11111110);
		spi_data8_2(1, 0b11111110);
		spi_data8_2(2, 0b11111110);
		spi_data8_2(3, 0b11111111);
	} else if((mode & 0xF0) == DISP_HTTP) {
		uint8_t i;
		if((sntpSec & 4) && memcmp(HttpMes+4, "\0\0\0\0", 4)) {
			for(i = 4; i < 8; i++)
				spi_data8_2(i-4, HttpMes[i] ^ (toggle & (HttpMesBlink>>i)));
		} else {
			for(i = 0; i < 4; i++)
				spi_data8_2(i-0, HttpMes[i] ^ (toggle & (HttpMesBlink>>i)));
		}
	} else if(mode == DISP_GPIO) {
		spi_data8_2(0, seg7dec[0]);		//'O'
		if(gpio_cur) {
			spi_data8_2(1, seg7alpha['N'-'A']);
			spi_data8_2(2, 0);
		} else {
			spi_data8_2(1, seg7alpha['F'-'A']);
			spi_data8_2(2, seg7alpha['F'-'A']);
		}
		spi_data8_2(3, 0);
	} else if((mode & 0xF0) == DISP_BAT) {
		switch(mode) {
		case DISP_BAT:
			batteryInit();
			DelayRoutine(10);
			d = batteryGet();
			batteryClose();
			break;

		case DISP_AD:
			d = ad10Get();
			break;
		}
		spi_data8_2(3, seg7dec[d%10]); d /= 10;
		spi_data8_2(2, seg7dec[d%10]); d /= 10;
		spi_data8_2(1, seg7dec[d%10]); d /= 10;
		spi_data8_2(0, seg7dec[d%10]);

	} else if(mode < DISP_EORZEA_HourMin) {
		const char WeekStr[8][4] = {"   ","SUN","MON","TUE","WED","THU","FRI","SAT"};
		switch(mode) {
		case DISP_HourMin:
			d = sntpSec/3600;
			spi_data8_2(1, seg7dec [d%10]); d = d / 10;
			spi_data8_2(0, seg7dec2[d%10]);

			d = (sntpSec/60)%60;
			spi_data8_2(3, seg7dec [d%10] | toggle); d = d / 10;
			spi_data8_2(2, seg7dec [d   ]);
			break;
		case DISP_Year:
			d = sntpYear;
			spi_data8_2(3, seg7dec [d%10]); d = d / 10;
			spi_data8_2(2, seg7dec [d%10]); d = d / 10;
			spi_data8_2(1, seg7dec [d%10]); d = d / 10;
			spi_data8_2(0, seg7dec [d%10]);
			break;
		case DISP_MonthDay:
			d = sntpMonth;
			spi_data8_2(1, seg7dec [d%10] | 1); d = d / 10;
			spi_data8_2(0, seg7dec2[d%10]);

			d = sntpDay;
			spi_data8_2(3, seg7dec [d%10]); d = d / 10;
			spi_data8_2(2, seg7dec2[d%10]);
			break;
		case DISP_Week:
			spi_data8_2(0, seg7alpha[WeekStr[sntpWeek][0]-'A']);
			spi_data8_2(1, seg7alpha[WeekStr[sntpWeek][1]-'A']);
			spi_data8_2(2, seg7alpha[WeekStr[sntpWeek][2]-'A'] | 1);
			spi_data8_2(3, 0);
			break;
		case DISP_Sec:
			d = sntpSec % 60;
			spi_data8_2(1, seg7dec [d%10]); d = d / 10;
			spi_data8_2(0, seg7dec [d   ]);

			spi_data8_2(2, 0);
			spi_data8_2(3, 0);
			break;
		}
	} else {
		#define UTC_DAY_OFFSET 17010
		#define EOR_MIN_OFFSET 503884800UL // 17010 * 24 * 3600 * 24 / 70
		uint32_t eorMin = ((uint32_t)(sntpDays - UTC_DAY_OFFSET)) * (24UL * 3600UL)
						+ sntpSec - (9UL * 3600UL);
		eorMin = (eorMin * 12UL) / 35UL + EOR_MIN_OFFSET;

		switch(mode) {
		case DISP_EORZEA_HourMin:
			d = (eorMin / 60UL) % 24;
			spi_data8_2(1, seg7dec [d%10]); d = d / 10;
			spi_data8_2(0, seg7dec2[d   ]);

			d = eorMin % 60;
			spi_data8_2(3, seg7dec [d%10] | toggle); d = d / 10;
			spi_data8_2(2, seg7dec [d   ]);
			break;
		case DISP_EORZEA_MonthDay:
			d = ((eorMin / (32UL*24UL*60UL)) % 12) + 1;
			spi_data8_2(1, seg7dec [d%10] | 1); d = d / 10;
			spi_data8_2(0, seg7dec2[d   ]);

			d = ((eorMin / (24UL*60UL)) % 32) + 1;
			spi_data8_2(3, seg7dec [d%10]); d = d / 10;
			spi_data8_2(2, seg7dec2[d   ]);
			break;
		case DISP_EORZEA_Year:
			d = eorMin / (12UL*32UL*24UL*60UL) + 1;
			spi_data8_2(3, seg7dec [d%10]); d = d / 10;
			spi_data8_2(2, seg7dec2[d%10]); d = d / 10;
			spi_data8_2(1, seg7dec2[d%10]); d = d / 10;
			spi_data8_2(0, seg7dec2[d%10]);
			break;
		case DISP_EORZEA_Week:
			d = ((eorMin / (24UL*60UL)) % 8) + 1;
			spi_data8_2(1, seg7dec [d   ]);

			spi_data8_2(0, 0);
			spi_data8_2(2, 0);
			spi_data8_2(3, 0);
			break;
		}
	}
}

static const uint8_t lcdMessage[3][3] = {
  {
	0b01101100,		// 'C'
	0b00011110,		// 'o'
	0b00011011,		// 'n.'
  },
  {
	0b01111100,		// 'E'
	0b00011000,		// 'r'
	0b00011001,		// 'r.'
  },
  {
	0b10101110,		// 'U'
	0b11111001,		// 'P.'
	0b00000000,		// ' '
  },
};

static void lcdDispMes(uint8_t index, uint8_t status)
{
	spi_data8(0, lcdMessage[index][0] | (1 & timeoutCnt & reqLowbat));
	spi_data8(1, lcdMessage[index][1]);
	spi_data8(2, lcdMessage[index][2]);
	spi_data8(3, seg7dec[status]);
}

//------------------------------------------------------------

static uint8_t upBufUart[CDC_DATA_IN_EP_SIZE] upBufUart_ADDRESS_TAG;
static uint8_t upBufSize;

static uint8_t downBufPos;
static uint8_t downBufSize;

static void CDCEmulatorInitialize()
{
	upBufSize = 0;
	downBufSize = 0;
}

static void CDCEmulatorTasks()
{
	// USB->UART(TX)
	if(downBufSize == 0 && !USBHandleBusy(CDCDataOutHandle)) {
		downBufSize = USBHandleGetLength(CDCDataOutHandle);
		if(downBufSize > 0) {
			downBufPos = 0;
		}
	}

	if(USBUSARTIsTxTrfReady() && (upBufSize > 0)) {
	//	PIE1bits.RCIE = 0;
		INTCONbits.GIEH = 0;
		memcpy(cdc_data_tx, upBufUart, upBufSize);
		mUSBUSARTTxRam((uint8_t*)cdc_data_tx, upBufSize);
		upBufSize = 0;
	//	PIE1bits.RCIE = 1;
		INTCONbits.GIEH = 1;
	}

	CDCTxService();
}

//------------------------------------------------------------

static uint8_t upBufUartLine[64];
static uint8_t upBufUartLineCnt;

unsigned long myStrtoul(char** dpp)
{
	if(*dpp == NULL) return 0;

	uint8_t d;
	while((d = **dpp) < '0' || d > '9') {
		if(d == '\0') {
			*dpp = NULL;
			return 0;
		}
		(*dpp)++;
	}

	unsigned long ret = 0;
	while((d = **dpp) >= '0' && d <= '9') {
		ret = ret * 10 + (d - '0');
		(*dpp)++;
	}
	return ret;
}

static void parseUartLine(void)
{
	if(upBufUartLineCnt < 3
	|| upBufUartLine[0] != 'S'
	|| upBufUartLine[1] < '1' || upBufUartLine[1] > '9'
	|| upBufUartLine[2] != ':')
		return;

	uint8_t lineIndex;
	lineIndex = upBufUartLine[1] - '0';

	uint8_t data;
	uint8_t i;

	char* dp = (char*)upBufUartLine+3;
	switch(lineIndex) {
	case STATUS_8_MODE:
		dispMode = myStrtoul(&dp);
		postMode = (dp && *dp == ' ') ? myStrtoul(&dp) : POST_OFF;
		break;

	case STATUS_7_HTTP:
		sntpStatus = SNTP_UPDATED;
		wifiStatus = STATUS_IDLE;
		if(postMode != POST_OFF)
			break;

		HttpMesBlink = 0;
		memset(HttpMes, 0, sizeof(HttpMes));

		for(i = 0; i < 8;) {
			data = *dp++;
			//24:20/10.10/09:33 06:33

			if(data == '/') {
				if(i <= 4 && *dp != '/') {
					i = 4;
				} else {
					i = 8;
				}
			} else if(data == ':' || data == ';') {
				if((i&3) == 2) {
					HttpMes[i+1] |= 1;		// 2->3,6->7
					if(data == ';') HttpMesBlink |= 1<<(i+1);
				}
			} else if(data == '.' || data == ',') {
				if((i&3) >= 2) {
					HttpMes[i-1] |= 1;		// 2->1,3->2, 6->5,7->6
					if(data == ',') HttpMesBlink |= 1<<(i-1);
				}
			} else {
					 if(data >= '0' && data <= '9')	data = seg7dec[data-'0'];
	 			else if(data == '-')				data = 0b00010000;	//-;
				else if(data >= 'a' && data <= 'z')	data = seg7alpha[data-'a'];
				else if(data >= 'A' && data <= 'Z')	data = seg7alpha[data-'A'];
				else 								data = 0;
				HttpMes[i++] |= data;
			}
		}

		data = myStrtoul(&dp);
		HttpMesNext = data*60 + myStrtoul(&dp);
		break;

	case STATUS_5_SNTP:
		// 2016/09/04 1 11:12:28 17048 40348
		for(data = 0; data < 9; data++) {
			unsigned long tmp = myStrtoul(&dp);
			switch(data) {
			case 0:	sntpYear  = tmp; break;
			case 1:	sntpMonth = tmp; break;
			case 2:	sntpDay   = tmp; break;
			case 3:	sntpWeek  = tmp; break;
		//	case 4:
		//	case 5:
		//	case 6:
			case 7:	sntpDays  = tmp; break;
			case 8:	sntpSec   = tmp; break;
			}
		}
		wifiStatus = lineIndex;
		if ((dispMode & 0xF0) == DISP_HTTP) {
			if(dp) sntpStatus = SNTP_NOTUPDATED;
		} else {
			if(dp) sntpStatus = SNTP_UPDATED;
			if(postMode == POST_OFF) {
				wifiStatus = STATUS_IDLE;
			}
		}
		break;

	case STATUS_6_ERROR:
		errorCode = ERROR_WPS;
		wifiStatus = lineIndex;
		break;

	default:
		wifiStatus = lineIndex;
		break;
	}
	if(lastLineIndex != lineIndex)
		timeoutCnt = 0;
	lastLineIndex = lineIndex;
	return;
}

void interrupt SYS_InterruptHigh(void)
{
	// timer1 (1sec)
#if LCDTYPE == 1	// MP rework
	if(INTCON3bits.INT2IF) {
		INTCON3bits.INT2IF = 0;
#else
	if(PIR1bits.TMR1IF) {
		TMR1H |= 0x80;
		PIR1bits.TMR1IF = 0;
#endif
		timeoutCnt++;
		if(timeoutWlan < TIMEOUT_WLAN)
			timeoutWlan++;

		if(sntpStatus)
			sntpSec++;
	}

	// UART RX
	if(PIR1bits.RCIF) {
		uint8_t data;
		data = USART_getcUSART();

		if(data == '\n') {
			upBufUartLine[upBufUartLineCnt] = '\0';
			parseUartLine();
			upBufUartLineCnt = 0;
		} else if(upBufUartLineCnt < sizeof(upBufUartLine)-1) {
			upBufUartLine[upBufUartLineCnt] = data;
			upBufUartLineCnt++;
		}

		if(upBufSize < sizeof(upBufUart)-1) {
			upBufUart[upBufSize] = data;
			upBufSize++;
		}
	}

	if(INTCONbits.INT0IF) {
		INTCONbits.INT0IF = 0;
		if(INTCONbits.INT0IE) {
			wifiStatus = STATUS_WLANOFF;
			INTCONbits.INT0IE = 0;			// INT0 external interrupt enable
			LATCbits.LATC5 = 0;				// WLAN EN=0
			reqLowbat = 1;
			if((dispMode & 0xF0) == DISP_HTTP)
				dispMode = DISP_HourMin;
			if(postMode != POST_OFF)
				postMode = POST_OFF;
		//	LATBbits.LATB4 = 1;		// RB4
		}
	}

	// PC1 wps
	if(INTCON3bits.INT1IF) {
		INTCON3bits.INT1IF = 0;
	//	if(wifiStatus != STATUS_1_WPSSTART)
	//		reqWps = 1;
	}

	// USB
	if(PIR2bits.USBIF)
		USBDeviceTasks();
}

void interrupt low_priority SYS_InterruptLow(void)
{

}

int checkUpdate(void)
{
	if(sntpStatus == SNTP_INVALID)
		return 0;

	#define SEC_PER_DAY (24UL * 3600UL)
	if(sntpSec >= SEC_PER_DAY) {
		sntpSec -= SEC_PER_DAY;
		goto Update;
	}

	if((sntpSec%60) != 0)
		return 0;

	if(postMode == POST_GPIO && gpio_last != gpio_cur) {
		goto Update;
	}

	uint16_t sntpMin = sntpSec/60;
	if(postMode == POST_AD && (sntpMin%60) == 0) {
		goto Update;
	}

	if(sntpStatus == SNTP_UPDATED) {
		if((dispMode & 0xF0) == DISP_HTTP && sntpMin == HttpMesNext)
			goto Update;
	} else {
		if((sntpMin % (3*60)) == 0)
			goto Update;
	}
	return 0;

Update:
	sntpStatus = SNTP_NOTUPDATED;
	return 1;
}

void main(void)
{
	OSCCON	= (0b00<<0)				// SCS: system clocl select
									//		00 = primary clock(determined by FOSC)
									//		01 = secondary oscillator
									//		1x = internal oscllator
			| (0<<2)				// HFIOFS: HFINTOSC frequency stable
									//		0 = HFINTOSC is no stable
									//		1 = HFINTOSC is stable
			| (0<<3)				// OSTS: oscillator start-up timeout status
									//		0 = device is running from the internal oscillator
									//		1 = device is running from the clock defined by FOSC
			| (0b011<<4)			// IRCF: internal oscillator frequency select
			| (0<<7);				// IDLEN: idle neable
									//		0 = device enter Sleep mode on SLEEP instruction
									//		1 = device enter Idle mode on SLEEP instruction
/*
	OSCCON2	= (0<<0)				// LFIOFS: LFINTOSC frequency stable
									//		0 = LFINTOSC is no stable
									//		1 = LFINTOSC is stable
			| (0<<1)				// HFIOFL: HFINTOSC frequency locked
			| (1<<2);				// PRO_SD: primary oscillator drive circuit shutdown
									//		0 = oscillator drive circuit off
*/

	ANSEL  = 0b00000000;			// AN7~3
	ANSELH = 0b00000000;			// AN11~8

	WPUA   = 0b00001000;			// WPUA5~3: Weak Pull-up enable bit
									// A3: (I)BOOT

	LATB  = 0b10000000;
	TRISB = 0b01110000;				// B4: (I)
									// B5: (I)RX
									// B6: (I)WLAN MODE
									// B7: (H)TX

	WPUB   = 0b00000000;			// WPUB7~4: Weak Pull-up enable bit

#if LCDTYPE == 1	// MP rework
	LATC  = 0b10011000;
	TRISC = 0b01000111;				// C0: (I)LOW BAT (INT0)
									// C1: (I)WPS (INT1)
									// C2: (I)IRQ (INT2)
									// C3: (H)CS
									// C4: (H)SDO
									// C5: (L)WLAN EN
									// C6: (I)VBUS/2
									// C7: (H)SCK(WR)

	RCONbits.IPEN = 1;

	INTCON3 = (0<<0)		// INT1IF: INT1 external interrupt flag
			| (0<<1)		// INT2IF: INT2 external interrupt flag
			| (1<<3)		// INT1IE: INT1 external interrupt enable
			| (1<<4)		// INT2IE: INT2 external interrupt enable
			| (1<<6)		// INT1IP: INT1 external interrupt priority
			| (1<<7);		// INT2IP: INT2 external interrupt priority
#else
	LATC  = 0b00011100;
	TRISC = 0b00000011;				// C0: (I)VBUS/2
									// C1: (I)WPS (INT1)
									// C2: (H)SCK(WR)
									// C3: (H)CS
									// C4: (H)SDO
									// C5: (L)WLAN EN
									// C6: (H)TOSCI
									// C7: (L)TOSCO

	// 32768Hz / 32768 / 1 = 1Hz

	TMR1H = 0x80;			// 32768
	TMR1L = 0x00;

	T1CON	= (1<<0)		// TMR1ON: Timer1 On bit
			| (1<<1)		// TMR1CS: Timer1 clock source select bit
							//		1 = External clock from T13CKI
							//		0 = internal clock(Fosc/4)
			| (1<<2)		// T1SYNC: External clock input sync select bit (TMR1CS=1)
							//		1 = do not synchronize external clock input
							//		0 = Synchronize exterenal clock input
			| (1<<3)		// T1OSCEN: Timer1 OSC enable
							//		1 = OSC is enabled
							//		0 = OSC is shut off
			| (0b00<<4)		// T1CKPS: Timer1 Clock Prescale
							//		00 = 1/1
							//		01 = 1/2
							//		10 = 1/4
							//		11 = 1/8
			| (0<<6)		// T1RUN: Timer1 System Clock Status
							//		1 - Main system clock is derived from Timer1 osc
							//		0 - Main system clock is derived from another source
			| (0<<7);		// RD16: 16bit R/W mode enable

	RCONbits.IPEN = 1;

	INTCON3 = (0<<0)		// INT1IF: INT1 external interrupt flag
			| (0<<1)		// INT2IF: INT2 external interrupt flag
			| (1<<3)		// INT1IE: INT1 external interrupt enable
			| (0<<4)		// INT2IE: INT2 external interrupt enable
			| (1<<6)		// INT1IP: INT1 external interrupt priority
			| (1<<7);		// INT2IP: INT2 external interrupt priority
#endif
	INTCON2	= (1<<0)		// RABIP: RA and RB port change interrupt priority
			| (1<<2)		// TMR0IP: TMR0 overflow interrupt priority
			| (0<<4)		// INTEDG2: External interrupt 2 edge select
							//		0 = interrupt on falling edge
							//		1 = interrupt on rising edge
			| (0<<5)		// INTEDG1: External interrupt 1 edge select
			| (0<<6)		// INTEDG0: External interrupt 0 edge select
			| (0<<7);		// xRABPU: PORTA and PORTB Pull-up enable

	INTCON	= (0<<0)		// RABIF: RA and RB port change interrupt flag
			| (0<<1)		// INT0IF: INT0 external interrupt flag
			| (0<<2)		// TMR0IF: TMR0 overflow interrupt flag
			| (0<<3)		// RABIE: RA and RB port change interrupt enable
			| (0<<4)		// INT0IE: INT0 external interrupt enable
			| (0<<5)		// TMR0IE: TMR0 overflow interrupt enable
			| (0<<6)		// GIEL: enable all low priorith interrupt
			| (1<<7);		// GIEH: enable all high priorith interrupt

	PIR1	= (0<<0)		// TMR1IF: TMR1 overflow interrupt flag
			| (0<<1)		// TMR2IF: TMR2 overflow interrupt flag
			| (0<<2)		// CCP1IF: CCP1 interrupt flag
			| (0<<3)		// SSPIF: MSSP interrupt flag
			| (0<<4)		// TXIF: EUSART transmit interrupt flag
			| (0<<5)		// RCIF: EUSART receive interrupt flag
			| (0<<6);		// ADIF: AD converter interrupt flag

	IPR1	= (1<<0)		// TMR1IP: TMR1 overflow interrupt priority
			| (1<<1)		// TMR2IP: TMR2 overflow interrupt priority
			| (1<<2)		// CCP1IP: CCP1 interrupt priority
			| (1<<3)		// SSPIP: MSSP interrupt priority
			| (1<<4)		// TXIP: EUSART transmit interrupt priority
			| (1<<5)		// RCIP: EUSART receive interrupt priority
			| (1<<6);		// ADIP: AD converter interrupt priority

#if LCDTYPE == 1	// MP rework
	PIE1	= (0<<0)		// TMR1IE: TMR1 overflow interrupt enable
			| (0<<1)		// TMR2IE: TMR2 overflow interrupt enable
			| (0<<2)		// CCP1IE: CCP1 interrupt enable
			| (0<<3)		// SSPIE: MSSP interrupt enable
			| (0<<4)		// TXIE: EUSART transmit interrupt enable
			| (1<<5)		// RCIE: EUSART receive interrupt enable
			| (0<<6);		// ADIE: AD converter interrupt enable
#else
	PIE1	= (1<<0)		// TMR1IE: TMR1 overflow interrupt enable
			| (0<<1)		// TMR2IE: TMR2 overflow interrupt enable
			| (0<<2)		// CCP1IE: CCP1 interrupt enable
			| (0<<3)		// SSPIE: MSSP interrupt enable
			| (0<<4)		// TXIE: EUSART transmit interrupt enable
			| (1<<5)		// RCIE: EUSART receive interrupt enable
			| (0<<6);		// ADIE: AD converter interrupt enable
#endif
	LATCbits.LATC5 = 1;		// C5: (H)WLAN EN
#if LCDTYPE == 1	// MP rework
	INTCONbits.INT0IE = 1;	// INT0 external interrupt enable
#endif

	USB_INTERRUPT_EN = 1;
	pCDCEmulatorInitialize = CDCEmulatorInitialize;
	USART_Initialize();
	USBDeviceInit();

	lcdInit();

	uint8_t vbus_filter = 0;
	uint8_t lastStatus = STATUS_0_BOOTING;
	uint8_t lastCnt = 0;
	uint8_t reqWps = 0;
	uint8_t runningWps = 0;
	uint8_t send0A = 0;
	uint8_t lastChar = 0x0a;
	uint8_t 
	enum {
		UPSTATE_IDLE = 0,
		UPSTATE_U,
		UPSTATE_U1,
		UPSTATE_U2,
		UPSTATE_INVALID = 0xFF,
	};
	uint8_t upState = UPSTATE_IDLE;

	sntpStatus = SNTP_INVALID;
	upBufUartLineCnt = 0;
	reqLowbat = 0;
	dispMode = 0;
	postMode = 0;
	wifiStatus = STATUS_0_BOOTING;
	lastLineIndex = 0;
	timeoutCnt = 0;
	timeoutWlan = 0;
	gpio_last = -1;
	memset(HttpMes, 0b00010000, 4);	// '-'

	WPUBbits.WPUB4 = 1;
	DelayRoutine(10);
	gpio_cur = PORTBbits.RB4 ^ 1;
	WPUBbits.WPUB4 = 0;

	// WLAN MODE
	if(PORTBbits.RB6 == 0) {
		wifiStatus = STATUS_WLANBOOT;
		lcdDispMes(MES_UP, 2);
	}

	int8_t i;
	while(1) {
		// USB
		if(USBGetDeviceState() >= CONFIGURED_STATE && USBIsDeviceSuspended()== false) {
			CDCEmulatorTasks();
		}

		// UART
		if(TXSTAbits.TRMT) {
			if(send0A == 1) {
				USART_putcUSART(0x0a);
				send0A = 0;
			} else if(downBufSize) {
				timeoutWlan = 0;

				char c = cdc_data_rx[downBufPos];
				if(wifiStatus == STATUS_WLANBOOT) {
					USART_putcUSART(c);
					const uint8_t WLAN_MAGIC[4] = {0xDE, 0xAA, 0xDE, 0xAA};
					if(c == WLAN_MAGIC[upState]) {
						upState++;
						if(upState == sizeof(WLAN_MAGIC)) {
							LATCbits.LATC5 = 0;				// WLAN EN=0
							USBDeviceDetach();
							OSCTUNEbits.SPLLEN = 0;
							DelayRoutine(1000);
							Reset();
							while(1);
						}
					} else {
						upState = UPSTATE_IDLE;
					}
				} else if(c == 0x0d || c == 0x0a) {
					switch(upState) {
					case UPSTATE_U:
						#define APP_VER_STR "\r\nPIC:" APP_VERSION_T "\r\n"
						memcpy(upBufUart, APP_VER_STR, sizeof(APP_VER_STR)-1);
						upBufSize = sizeof(APP_VER_STR)-1;
						break;
					case UPSTATE_U1:
						INTCON = 0x00;
						wifiStatus = STATUS_WLANOFF;
						LATCbits.LATC5 = 0;				// WLAN EN=0

						USBDeviceDetach();
						OSCTUNEbits.SPLLEN = 0;

						// 3sec wait
						for(i = 0; i < 4;) {
						#if LCDTYPE == 1	// MP rework
							if(INTCON3bits.INT2IF) {
								INTCON3bits.INT2IF = 0;
						#else
							if(PIR1bits.TMR1IF) {
								TMR1H |= 0x80;
								PIR1bits.TMR1IF = 0;
						#endif
								i++;
							}
						}
					#asm
						goto 0x30
					#endasm
						break;
					case UPSTATE_U2:		// 'U2'
						wifiStatus = STATUS_WLANOFF;
					#if LCDTYPE == 1	// MP rework
						INTCONbits.INT0IE = 0;			// INT0 external interrupt enable
					#endif
						LATCbits.LATC5 = 0;				// WLAN EN=0
						LATBbits.LATB6 = 0;
						TRISBbits.TRISB6 = 0;
						lcdDispMes(MES_UP, 2);
						DelayRoutine(1000);

						LATCbits.LATC5 = 1;				// WLAN EN=1
					#if LCDTYPE == 1	// MP rework
						INTCONbits.INT0IE = 1;			// INT0 external interrupt enable
					#endif
						wifiStatus = STATUS_WLANBOOT;
						break;
					}
					upState = UPSTATE_IDLE;

					if(lastChar == 0x0d && c == 0x0a) {
						;
					} else {
						USART_putcUSART(0x0d);
						send0A = 1;
						lastLineIndex = 0;
					}
				} else {
					USART_putcUSART(c);
					switch(upState) {
					case UPSTATE_IDLE:
						upState =	(c == 'U') ? UPSTATE_U:
									UPSTATE_INVALID;
						break;
					case UPSTATE_U:
						upState =	(c == '1') ? UPSTATE_U1 : 
									(c == '2') ? UPSTATE_U2: 
									UPSTATE_INVALID;
						break;
					}
				}
				lastChar = c;
				downBufPos++;
				if (downBufPos >= downBufSize) {
					getsUSBUSART();
					downBufSize = 0;
				}
			}
		}

		// wifi state
		if(lastStatus != wifiStatus) {
			switch(wifiStatus) {
			case STATUS_5_SNTP:
				if(postMode != POST_OFF) {
					#define CMD_POST "\r\nATPOST="

					batteryInit();
					DelayRoutine(10);
					memcpy(cdc_data_rx+0, CMD_POST, sizeof(CMD_POST)-1);
					itoa((char*)cdc_data_rx + sizeof(CMD_POST)-1, batteryGet(), 10);
					batteryClose();

					int len;
					len = strlen((char*)cdc_data_rx);

					uint16_t val = 0;
					switch(postMode) {
					case POST_GPIO:
						if(gpio_last == gpio_cur) {
							sntpStatus = SNTP_UPDATED;
							wifiStatus = STATUS_IDLE;
							goto skip;
						}
						gpio_last = gpio_cur;
						val = gpio_cur;
						break;

					case POST_AD:
						val = ad10Get();
						break;
					}

					if(postMode != POST_BAT) {
						cdc_data_rx[len] = ' ';
						itoa((char*)cdc_data_rx + len + 1, val, 10);
						len = strlen((char*)cdc_data_rx);
					}
					memcpy(cdc_data_rx + len, "\r\n", 3);
					downBufSize = len + 3;
					downBufPos = 0;
				} else if((dispMode & 0xF0) == DISP_HTTP) {
					#define CMD_HTTP "\r\nATHTTP\r\n"
					memcpy(cdc_data_rx+0, CMD_HTTP, sizeof(CMD_HTTP)-1);
					downBufSize = sizeof(CMD_HTTP)-1;
					downBufPos = 0;
				} else {
				}
skip:
			case STATUS_2_CONNECTING:
			case STATUS_3_CONNECTED:
			case STATUS_4_GOTIP:
				if(reqWps && runningWps) {
					reqWps = 0;
					runningWps = 0;
				}
			case STATUS_1_WPSSTART:
				lcdDispMes(MES_CON, wifiStatus);
				break;

			case STATUS_IDLE:
				lcdUpdateDate();
				break;

			case STATUS_6_ERROR:
				runningWps = 0;
				lcdDispMes(MES_ERR, errorCode);
				break;

			default:
				break;
			}
			lastStatus = wifiStatus;
		}

		// WPS
		if(PORTCbits.RC1 == 0 && wifiStatus != STATUS_1_WPSSTART)
			reqWps = 5;		// 15sec * 5

		if(lastCnt == timeoutCnt)
			continue;
		lastCnt = timeoutCnt;

		//-----------------------------------------------------------
		// 1sec interval

		// VBUS
#if LCDTYPE == 1	// MP rework
		vbus_filter = ((vbus_filter<<1) | PORTCbits.RC6) & 0b11;
#else
		vbus_filter = ((vbus_filter<<1) | PORTCbits.RC0) & 0b11;
#endif
		if((reqLowbat == 1 || vbus_filter == 0b00) && OSCTUNEbits.SPLLEN == 1) {
		//	UCFGbits.UPUEN = 0;
			USBDeviceDetach();
			OSCTUNEbits.SPLLEN = 0;
			SPBRG  =  SPBRG_12M;
			SPBRGH = (SPBRG_12M>>8);

		} else if(reqLowbat == 0 && vbus_filter == 0b11 && OSCTUNEbits.SPLLEN == 0) {
			OSCTUNEbits.SPLLEN = 1;	// 4 x pll enable
			SPBRG  =  SPBRG_48M;
			SPBRGH = (SPBRG_48M>>8);
			timeoutWlan = 0;
			USBDeviceAttach();
		//	UCFGbits.UPUEN = 1;
		}

		// WPS
		if(reqWps && !runningWps) {
			switch(wifiStatus) {
			case STATUS_0_BOOTING:
			case STATUS_WLANOFF:
				break;

			default:
				#define CMD_WPS "\r\nATWPS\r\n"
				memcpy(cdc_data_rx+0, CMD_WPS, sizeof(CMD_WPS)-1);
				downBufSize = sizeof(CMD_WPS)-1;
				downBufPos = 0;

				reqWps--;
				runningWps = 1;
				wifiStatus = STATUS_1_WPSSTART;
				timeoutCnt = 0;
				break;
			}
		}

		switch(wifiStatus) {
		case STATUS_0_BOOTING:
			if(timeoutCnt == 10) {
				memcpy(cdc_data_rx+0, "\r\n", 2);
				downBufSize = 2;
			} else if(timeoutCnt > 20) {
				errorCode = ERROR_WLANFW;
				wifiStatus = STATUS_6_ERROR;
			}
			break;

		case STATUS_1_WPSSTART:
		case STATUS_2_CONNECTING:
		case STATUS_3_CONNECTED:
		case STATUS_4_GOTIP:
		case STATUS_5_SNTP:
			if(timeoutCnt > 30) {
				errorCode = (wifiStatus == STATUS_5_SNTP) ? ERROR_7_HTTP : wifiStatus;
				wifiStatus = STATUS_6_ERROR;
				timeoutCnt = 0;
			}
			break;

		case STATUS_6_ERROR:
			if(timeoutCnt > 10 && errorCode != ERROR_WLANFW) {
				wifiStatus = STATUS_IDLE;
			}
			break;

		case STATUS_IDLE:
			if(checkUpdate() && reqLowbat == 0) {
				wifiStatus = STATUS_4_GOTIP;
				timeoutCnt = 0;
				lcdDispMes(MES_CON, wifiStatus);
				#define CMD_RST "\r\nAT+RST\r\n"
				memcpy(cdc_data_rx+0, CMD_RST, sizeof(CMD_RST)-1);
				downBufSize = sizeof(CMD_RST)-1;
				downBufPos = 0;
			} else {
				lcdUpdateTime();

				if(!(OSCTUNEbits.SPLLEN == 1 && timeoutWlan < TIMEOUT_WLAN)) {
					if(OSCTUNEbits.SPLLEN == 1) {
						#define SLEEP_MES "\r\nPIC:enter to WLAN sleep.\r\n"
						memcpy(upBufUart, SLEEP_MES, sizeof(SLEEP_MES)-1);
						upBufSize = sizeof(SLEEP_MES)-1;
					}
					wifiStatus = STATUS_WLANOFF;
				#if LCDTYPE == 1	// MP rework
					INTCONbits.INT0IE = 0;			// INT0 external interrupt enable
				#endif
					LATCbits.LATC5 = 0;				// WLAN EN=0
				}
			}
			break;

		case STATUS_WLANOFF:
			if((checkUpdate() || (OSCTUNEbits.SPLLEN == 1 && timeoutWlan < TIMEOUT_WLAN) || reqWps > 0) && reqLowbat == 0) {
				LATCbits.LATC5 = 1;				// WLAN EN=1
			#if LCDTYPE == 1	// MP rework
				INTCONbits.INT0IE = 1;			// INT0 external interrupt enable
			#endif
				wifiStatus = STATUS_0_BOOTING;
				timeoutCnt = 0;
				lcdDispMes(MES_CON, 0);
			} else {
				lcdUpdateTime();
				if(OSCTUNEbits.SPLLEN == 0) {
					SLEEP();
					NOP();
				}
			}
			break;

		case STATUS_WLANBOOT:
			if(PORTBbits.RB6 == 1) {
			//	reset();
			}
			break;

		default:
			break;
		}
	}
}
