//DOM-IGNORE-BEGIN
/*******************************************************************************
Software License Agreement

The software supplied herewith by Microchip Technology Incorporated
(the "Company") for its PICmicro(R) Microcontroller is intended and
supplied to you, the Company's customer, for use solely and
exclusively on Microchip PICmicro Microcontroller products. The
software is owned by the Company and/or its supplier, and is
protected under applicable copyright laws. All rights are reserved.
Any use in violation of the foregoing restrictions may subject the
user to criminal sanctions under applicable laws, as well as to
civil liability for the breach of the terms and conditions of this
license.

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.

*******************************************************************************/
//DOM-IGNORE-END

#ifndef _USB_HAL_PIC18_H
#define _USB_HAL_PIC18_H

// *****************************************************************************
// *****************************************************************************
// Section: Included Files
// *****************************************************************************
// *****************************************************************************
/*  This section lists the other files that are included in this file.
*/
#include <xc.h>
#include <stdint.h>

#include <string.h>

#include "system.h"
#include "system_config.h"

#ifdef __cplusplus  // Provide C++ Compatability
    extern "C" {
#endif

// *****************************************************************************
// *****************************************************************************
// Section: Constants
// *****************************************************************************
// *****************************************************************************

//----- USBEnableEndpoint() input defintions ----------------------------------
#define USB_HANDSHAKE_ENABLED   0x10
#define USB_HANDSHAKE_DISABLED  0x00

#define USB_OUT_ENABLED         0x04
#define USB_OUT_DISABLED        0x00

#define USB_IN_ENABLED          0x02
#define USB_IN_DISABLED         0x00

#define USB_ALLOW_SETUP         0x00
#define USB_DISALLOW_SETUP      0x08

#define USB_STALL_ENDPOINT      0x01

//----- Event call back defintions --------------------------------------------
#if defined(USB_DISABLE_SOF_HANDLER)
    #define USB_SOF_INTERRUPT 0x00
#else
    #define USB_SOF_INTERRUPT 0x40
#endif

#if defined(USB_DISABLE_ERROR_HANDLER)
    #define USB_ERROR_INTERRUPT 0x02
#else
    #define USB_ERROR_INTERRUPT 0x02
#endif

//----- BDnSTAT bit definitions -----------------------------------------------
#define _BSTALL     0x04        //Buffer Stall enable
#define _DTSEN      0x08        //Data Toggle Synch enable
#define _DAT0       0x00        //DATA0 packet expected next
#define _DAT1       0x40        //DATA1 packet expected next
#define _DTSMASK    0x40        //DTS Mask
#define _USIE       0x80        //SIE owns buffer
#define _UCPU       0x00        //CPU owns buffer
#define _STAT_MASK  0xFF

#define USTAT_EP0_PP_MASK   ~0x02
#define USTAT_EP_MASK       0x7E
#define USTAT_EP0_OUT       0x00
#define USTAT_EP0_OUT_EVEN  0x00
#define USTAT_EP0_OUT_ODD   0x02
#define USTAT_EP0_IN        0x04
#define USTAT_EP0_IN_EVEN   0x04
#define USTAT_EP0_IN_ODD    0x06

#define ENDPOINT_MASK 0b01111000

//----- U1EP bit definitions --------------------------------------------------
#define UEP_STALL 0x0001
// Cfg Control pipe for this ep
/* Endpoint configuration options for USBEnableEndpoint() function */
#define EP_CTRL     0x06            // Cfg Control pipe for this ep
#define EP_OUT      0x0C            // Cfg OUT only pipe for this ep
#define EP_IN       0x0A            // Cfg IN only pipe for this ep
#define EP_OUT_IN   0x0E            // Cfg both OUT & IN pipes for this ep

//----- Defintions for BDT address --------------------------------------------
#if defined(__18F14K50) || defined(__18F13K50) || defined(__18LF14K50) || defined(__18LF13K50)
    #define USB_BDT_ADDRESS 0x200     //See Linker Script, BDT in bank 2 on these devices - usb2:0x200-0x2FF(256-byte)
#endif

#if (USB_PING_PONG_MODE == USB_PING_PONG__NO_PING_PONG)
    #define BDT_NUM_ENTRIES      ((USB_MAX_EP_NUMBER + 1) * 2)
#elif (USB_PING_PONG_MODE == USB_PING_PONG__EP0_OUT_ONLY)
    #define BDT_NUM_ENTRIES      (((USB_MAX_EP_NUMBER + 1) * 2)+1)
#elif (USB_PING_PONG_MODE == USB_PING_PONG__FULL_PING_PONG)
    #define BDT_NUM_ENTRIES      ((USB_MAX_EP_NUMBER + 1) * 4)
#elif (USB_PING_PONG_MODE == USB_PING_PONG__ALL_BUT_EP0)
    #define BDT_NUM_ENTRIES      (((USB_MAX_EP_NUMBER + 1) * 4)-2)
#else
    #error "No ping pong mode defined."
#endif

#if defined(__XC8)
	#define BDT_BASE_ADDR_TAG		@USB_BDT_ADDRESS
	#define CTRL_TRF_SETUP_ADDR_TAG	@(USB_BDT_ADDRESS+(BDT_NUM_ENTRIES*4))
	#define CTRL_TRF_DATA_ADDR_TAG	@(USB_BDT_ADDRESS+(BDT_NUM_ENTRIES*4)+USB_EP0_BUFF_SIZE)
#endif

//----- Depricated defintions - will be removed at some point of time----------
//--------- Depricated in v2.2
#define _LS         0x00            // Use Low-Speed USB Mode
#define _FS         0x04            // Use Full-Speed USB Mode
#define _TRINT      0x00            // Use internal transceiver
#define _TREXT      0x08            // Use external transceiver
#define _PUEN       0x10            // Use internal pull-up resistor
#define _OEMON      0x40            // Use SIE output indicator

// Buffer Descriptor Status Register layout.
typedef union _BD_STAT
{
	uint8_t Val;
	struct{
		//If the CPU owns the buffer then these are the values
		unsigned _0:1;
		unsigned _1:1;
		unsigned BSTALL:1;		//Buffer Stall Enable
		unsigned DTSEN:1;		//Data Toggle Synch Enable
		unsigned _4:1;
		unsigned _5:1;
		unsigned DTS:1;			//Data Toggle Synch Value
		unsigned UOWN:1;		//USB Ownership
	};
	struct{
		unsigned :2;
		unsigned PID:4;			//Packet Identifier
		unsigned :2;
	};
} BD_STAT;						//Buffer Descriptor Status Register

// BDT Entry Layout
typedef union __BDT
{
	struct
	{
		BD_STAT		STAT;
		uint8_t		CNT;
		uint16_t	ADR;
	};
	uint32_t Val;
} BDT_ENTRY;

// USTAT Register Layout
typedef union __USTAT
{
	struct
	{
		unsigned char filler1:1;
		unsigned char ping_pong:1;
		unsigned char direction:1;
		unsigned char endpoint_number:4;
	};
	uint8_t Val;
} USTAT_FIELDS;

//Macros for fetching parameters from a USTAT_FIELDS variable.
#define USBHALGetLastEndpoint(stat)     stat.endpoint_number
#define USBHALGetLastDirection(stat)    stat.direction
#define USBHALGetLastPingPong(stat)     stat.ping_pong
/*
typedef union _POINTER
{
	uint8_t* bRam;            // Ram byte pointer: 2 bytes pointer pointing to 1 byte of data
	uint16_t* wRam;           // Ram word poitner: 2 bytes poitner pointing to 2 bytes of data
	const uint8_t* bRom;      // Size depends on compiler setting
	const uint16_t* wRom;
} POINTER;
*/
#define ConvertToPhysicalAddress(a) ((uint16_t)(a))
#define ConvertToVirtualAddress(a)  ((void *)(a))

extern uint8_t USB_INTERRUPT_EN;
#define USBClearUSBInterrupt() {PIR2bits.USBIF = 0;}
#if defined(USB_INTERRUPT)
    #define USBMaskInterrupts() {PIE2bits.USBIE = 0;}
    #define USBUnmaskInterrupts() {PIE2bits.USBIE = USB_INTERRUPT_EN;}
#else
    #define USBMaskInterrupts()
    #define USBUnmaskInterrupts()
#endif

#define USBInterruptFlag PIR2bits.USBIF

//STALLIE, IDLEIE, TRNIE, and URSTIE are all enabled by default and are required
#if defined(USB_INTERRUPT)
    #define USBEnableInterrupts() {IPR2bits.USBIP = 1;PIE2bits.USBIE = USB_INTERRUPT_EN;}
#else
    #define USBEnableInterrupts()
#endif

#define USBDisableInterrupts() {PIE2bits.USBIE = 0;}

#define SetConfigurationOptions()   {\
	UCFG = USB_PULLUP_OPTION | USB_TRANSCEIVER_OPTION | USB_SPEED_OPTION | USB_PING_PONG_MODE;\
	UEIE = 0x9F;\
	UIE = 0x39 | USB_SOF_INTERRUPT | USB_ERROR_INTERRUPT;\
}

#define USBModuleDisable() {\
    UCON = 0;\
    UIE = 0;\
    USBDeviceState = DETACHED_STATE;\
}

// Clears the control registers for the specified non-zero endpoints
#define DisableNonZeroEndpoints(last_ep_num) memset((void*)&UEP1,0x00,(last_ep_num));

#ifndef USB_PING_PONG_MODE
    #error "No ping pong mode defined."
#endif

#ifdef __cplusplus  // Provide C++ Compatibility
    }
#endif

#endif //#ifndef _USB_HAL_PIC18_H
